"""airplane - An SDK for writing Airplane tasks in Python"""

__version__ = "0.2.3"
__all__ = ["Airplane"]

import os

from .client import Airplane

default_client = None

_api_host = os.getenv("AP_TASK_RUNTIME_API_HOST")
_api_token = os.getenv("AP_TASK_RUNTIME_TOKEN")


def write_output(value):
    """Writes the value to the task's output."""
    return _proxy("write_output", value)


def write_named_output(name, value):
    """Writes the value to the task's output, tagged by the key."""
    return _proxy("write_named_output", name, value)


def run(task_id, parameters, env={}, constraints={}):
    """Triggers an Airplane task with the provided arguments."""
    return _proxy("run", task_id, parameters, env=env, constraints=constraints)


def _proxy(method, *args, **kwargs):
    global default_client
    if not default_client:
        default_client = Airplane(_api_host, _api_token)

    fn = getattr(default_client, method)
    return fn(*args, **kwargs)



import json
import os
import requests
import backoff
from requests.models import HTTPError

from .exceptions import InvalidEnvironmentException, RunPendingException


class Airplane:
    """Client SDK for Airplane tasks."""

    def __init__(self, api_host, api_token):
        self._api_host = api_host
        self._api_token = api_token

    def write_output(self, value):
        """Writes the value to the task's output."""
        val = json.dumps(value, separators=(",", ":"))
        print(f"airplane_output {val}")

    def write_named_output(self, name, value):
        """Writes the value to the task's output, tagged by the key."""
        val = json.dumps(value, separators=(",", ":"))
        print(f"airplane_output:\"{name}\" {val}")

    def run(self, task_id, parameters, env={}, constraints={}):
        """Triggers an Airplane task with the provided arguments."""
        self.__require_runtime()

        # Boot the new task:
        resp = requests.post(
            f"{self._api_host}/v0/runs/create",
            json={
                "taskID": task_id,
                "params": parameters,
                "env": env,
                "constraints": constraints,
            },
            headers={
                "X-Airplane-Token": self._api_token,
            },
        )
        self.__check_resp(resp)
        body = resp.json()
        run_id = body["runID"]

        return self.__wait(run_id)

    def __check_resp(self, resp):
        if resp.status_code >= 400:
            raise HTTPError(resp.json()["error"])

    def __require_runtime(self):
        """Ensures that the current task is running inside of an Airplane task."""
        if self._api_host is None or self._api_token is None:
            raise InvalidEnvironmentException()

    def __backoff():
        yield from backoff.expo(factor=0.1, max_value=5)

    @backoff.on_exception(
        __backoff,
        (
            requests.exceptions.ConnectionError,
            requests.exceptions.Timeout,
            RunPendingException,
        ),
        max_tries=1000,
    )
    def __wait(self, run_id):
        resp = requests.get(
            f"{self._api_host}/v0/runs/get",
            params={"runID": run_id},
            headers={
                "X-Airplane-Token": self._api_token,
            },
        )
        self.__check_resp(resp)
        body = resp.json()
        run_status = body["run"]["status"]

        if run_status in ("NotStarted", "Queued", "Active"):
            # Retry...
            raise RunPendingException()

        resp = requests.get(
            f"{self._api_host}/v0/runs/getOutputs",
            params={"runID": run_id},
            headers={
                "X-Airplane-Token": self._api_token,
            },
        )
        self.__check_resp(resp)
        body = resp.json()

        return {"status": run_status, "outputs": body["outputs"]}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['airplane']

package_data = \
{'': ['*']}

install_requires = \
['backoff>=1.10.0,<2.0.0', 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'airplanesdk',
    'version': '0.2.3',
    'description': 'A Python SDK for writing Airplane tasks',
    'long_description': '# Airplane Python SDK [![PyPI](https://img.shields.io/pypi/v/airplanesdk)](https://pypi.org/project/airplanesdk/) [![PyPI - License](https://img.shields.io/pypi/l/airplanesdk)](./LICENSE) [![Docs](https://img.shields.io/badge/Docs-Python%20SDK-blue)](https://docs.airplane.dev/reference/runtime-api-and-airplane-sdk/python-sdk)\n\nAn SDK for writing Airplane tasks in Python.\n\n## Usage\n\nFirst, install the SDK:\n\n```sh\npip install airplanesdk\n```\n\nNext, you can use the SDK to produce outputs which will be separated from your logs:\n\n```python\nimport airplane\n\nairplane.write_output("Show me what you got")\n\n# You can also separate outputs into groups by attaching names:\nairplane.write_named_output("saying", "Show me what you got")\nairplane.write_named_output("saying", "Welcome to the club, pal")\nairplane.write_named_output("name", "Summer")\n```\n\nThis SDK can be used to programmatically kick off tasks and fetch their output:\n\n```python\n# You can get a task\'s ID from the URL bar, f.e.\n# https://app.airplane.dev/tasks/1oMt2mZC1DjkOZXxHH8BV57xrmF\ntask_id = "..."\nresp = airplane.run(task_id, {\n  # Optionally provide parameters to your task, using the same name\n  # as when templating a parameter into your task\'s CLI args.\n  "DryRun": True,\n})\n\n# run() will return the run\'s status (Succeeded, Failed, Cancelled) and a\n# dict of outputs, by name.\n#\n# Default outputs are available as `resp["outputs"]["output"]`.\nprint(resp["outputs"])\n```\n',
    'author': 'Airplane',
    'author_email': 'support@airplane.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://airplane.dev',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
